{% extends 'appearance/base.html' %}

{% load views_tags %}

{% block title %}{% include 'appearance/partials/form_title.html' with non_html_title=True %}{% endblock %}

{% block content %}
    <div class="row">
        {% for subtemplate in subtemplates_list %}
            {% if subtemplate.column_class %}
                <div class="{{ subtemplate.column_class }}">
            {% else %}
                <div class="cols-xs-12">
            {% endif %}
                {% if subtemplate.form %}
                    {% views_render_subtemplate subtemplate.name subtemplate.context as rendered_subtemplate %}
                        {{ rendered_subtemplate }}
                {% else %}
                    {% views_render_subtemplate subtemplate.name subtemplate.context as rendered_subtemplate %}
                    {{ rendered_subtemplate }}
                {% endif %}
            </div>
         {% endfor %}
    </div>
{% endblock %}
