{% load static %}

{% load smart_settings_tags %}

<link href="{% static 'documents/css/documents.css' %}" media="screen" rel="stylesheet" type="text/css" />

{% smart_setting global_name='DOCUMENTS_LIST_THUMBNAIL_WIDTH' as thumbnail_width %}

<style>
    .document-thumbnail-list div.text-center div.fa-4x span.fa-layers {
        font-size: {% if thumbnail_width < '50' %}{{ thumbnail_width }}{% else %}50{% endif %}px;
        {#font-size: {{ thumbnail_width }}px;#}
        margin: 0px;
        margin-left: -5px;
    }
</style>
